# Learning anti-classes with one-cold cross entropy loss (Demo Code)

## Overview

This repository contains the demo code for our ICLR 2025 submission. The code is anonymized and includes scripts for reproducing the main closed set classification experiments.

## Requirements

The code is tested with the following environment:
- Python 3.8
- PyTorch 2.3.0+cu118

To install the required dependencies, use the provided `requirements.txt` file:

```bash
pip install -r requirements.txt
```

## Training the Model

To train the model, use the `train.py` script with the following commands:

### Baseline Cross Entropy

```bash
python train.py --dataset 'cifar100' --w_ce=1.0 --w_occe=0.0 --gpu 0 --seed 0
```

### Including the OCCE Loss

```bash
python train.py --dataset 'cifar100' --w_ce=1.0 --w_occe=1.0 --gpu 0 --seed 0
```

## Pre-trained Logs

The tensorboard logs for the training configurations with 3 different seeds are available in the `logs` folder.

## Plotting Accuracy Curves

To plot the accuracy curves from these logs, use the `plot.py` script:

```bash
python plot.py
```

## Demo Script

You can also run the provided shell script `demo.sh`, which trains the baseline and the proposed models from scratch for 3 seeds with the specified configuration:

```bash
sh demo.sh
```

## Notes

- Ensure you have set up your environment correctly with Python 3.8 and the specified PyTorch version.
- All scripts assume the availability of a GPU, which can be specified using the `--gpu` flag.

For any further details, please refer to the inline comments within the scripts.

---


